@@ -94,6 +94,9 @@ static gboolean oc_get_time(Clock *clock)
             utf8_strftime(res, sizeof(res), line->data->str, &clock->now);
             /* gtk_label_set_text call takes almost
              * 100 % of the time wasted in this procedure 
+             * Note that even though we only wake up when needed, we 
+             * may not have to update all lines, so this check still
+             * probably is worth doing
              * */
             if (strcmp(res,  line->prev)) {
                 gtk_label_set_text(GTK_LABEL(line->label), res);
@@ -106,6 +109,196 @@ static gboolean oc_get_time(Clock *clock)
     return(TRUE);
 }
 
+static gboolean oc_get_time_delay(Clock *clock)
+{
+    clock->timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE
+            , clock->interval, (GSourceFunc)oc_get_time, clock, NULL);
+    oc_get_time(clock);
+    return(FALSE); /* this is one time only timer */
+}
+
+gboolean oc_start_timer(Clock *clock)
+{
+    time_t t;
+    gint   delay_time =  0;
+    /* if we are using longer than 1 minute (= 60000) interval, we need
+     * to delay the first start so that clock changes when minute or hour
+     * changes */
+
+    oc_get_time(clock);
+    time(&t);
+    localtime_r(&t, &clock->now);
+    if (clock->interval >= 60000) {
+        if (clock->interval >= 3600000) /* match to next full hour */
+            delay_time = (clock->interval -
+                    (clock->now.tm_min*60000 + clock->now.tm_sec*1000));
+        else /* match to next full minute */
+            delay_time = (clock->interval - clock->now.tm_sec*1000);
+    }
+    if (clock->delay_timeout_id) {
+        g_source_remove(clock->delay_timeout_id);
+        clock->delay_timeout_id = 0;
+    }
+    if (clock->timeout_id) {
+        g_source_remove(clock->timeout_id);
+        clock->timeout_id = 0;
+    }
+    clock->delay_timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE
+            , delay_time, (GSourceFunc)oc_get_time_delay, clock, NULL);
+    /* if we have longer than 1 sec timer, we need to reschedule 
+     * it regularly since it will fall down slowly but surely, so
+     * we keep this running. */
+    if (clock->interval >= 60000) {
+        if (delay_time > 60000)
+        /* let's run it once in case we happened to kill it
+           just when it was supposed to start */
+            oc_get_time(clock); 
+        return(FALSE);
+    }
+    else
+        return(TRUE);
+}
+
+static void oc_end_tuning(Clock *clock)
+{
+    /* if we have longer than 1 sec timer, we need to reschedule 
+     * it regularly since it will fall down slowly but surely */
+    if (clock->adjust_timeout_id) {
+        g_source_remove(clock->adjust_timeout_id);
+        clock->adjust_timeout_id = 0;
+    }
+    if (clock->interval >= 60000) { /* resync it after each 6 hours */
+        clock->adjust_timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE
+                , 6*60*60*1000, (GSourceFunc)oc_start_timer, clock, NULL);
+    }
+    g_free(clock->tune);
+    clock->tune = NULL;
+}
+
+static gboolean oc_next_level_tuning(Clock *clock)
+{
+    TimeTuning *tune = clock->tune;
+    gboolean continue_tuning = TRUE;
+
+    if (clock->interval == OC_CONFIG_INTERVAL) /* setup active => no changes */
+        return(FALSE);
+    switch (tune->level) {
+        case 0: /* second now; let's set interval to minute */
+            clock->interval = 60*1000; 
+            break;
+        case 1: /* minute now; let's set it to hour */
+            clock->interval = 60*60*1000; 
+            break;
+        default: /* no point tune more */
+            break;
+    }
+    if (tune->level < 2) {
+        tune->level++;
+        oc_start_timer(clock);
+        oc_start_tuning(clock);
+    }
+    else {
+        continue_tuning = FALSE;
+        oc_end_tuning(clock);
+    }
+    return(continue_tuning);
+}
+
+static gboolean oc_tune_time(Clock *clock)
+{
+    /* idea is that we compare the panel string three times and 
+     * if all of those are the same, we can tune the timer since we know 
+     * that we do not have to check it so often. Time in panel plugin is
+     * not changing anyway. So we adjust the timers and check again */
+    gboolean continue_tuning = TRUE;
+    TimeTuning *tune = clock->tune;
+
+    if (clock->interval == OC_CONFIG_INTERVAL) /* setup active => no changes */
+        return(FALSE);
+    switch (tune->cnt) {
+        case 0: /* first time we come here */
+            strncpy(tune->prev[0], clock->line[0].prev, OC_MAX_LINE_LENGTH);
+            strncpy(tune->prev[1], clock->line[1].prev, OC_MAX_LINE_LENGTH);
+            strncpy(tune->prev[2], clock->line[2].prev, OC_MAX_LINE_LENGTH);
+            strncpy(tune->tooltip_prev, clock->tooltip_prev,OC_MAX_LINE_LENGTH);
+            break;
+        case 1:
+        case 2:
+        case 3: /* FIXME: two may be enough ?? */
+            if (strncmp(tune->prev[0], clock->line[0].prev, OC_MAX_LINE_LENGTH)
+            ||  strncmp(tune->prev[1], clock->line[1].prev, OC_MAX_LINE_LENGTH)
+            ||  strncmp(tune->prev[2], clock->line[2].prev, OC_MAX_LINE_LENGTH)
+            ||  strncmp(tune->tooltip_prev, clock->tooltip_prev
+                    ,OC_MAX_LINE_LENGTH)) {
+                /* time has changed. But it may change due to bigger time
+                 * change. For example if we are checking seconds and time
+                 * only shows minutes, this can change if hour happens to
+                 * change. So we need to check the change twice to be sure */
+                if (tune->changed_once) { /* this is the second trial already */
+                    continue_tuning = FALSE;
+                    oc_end_tuning(clock);
+                }
+                else { /* need to try again */
+                    tune->changed_once = TRUE;
+                    tune->cnt = -1; /* this gets incremented to 0 below */
+                }
+            }
+            break;
+        default:
+            break;
+    }
+    if (continue_tuning) {
+        if (tune->cnt == 3) 
+            continue_tuning = oc_next_level_tuning(clock);
+        else 
+            tune->cnt++;
+    }
+
+    return(continue_tuning);
+}
+
+void oc_disable_tuning(Clock *clock)
+{
+    TimeTuning *tune = clock->tune;
+
+    if (tune == NULL)
+        return;
+    if (tune->timeout_id) {
+        g_source_remove(tune->timeout_id);
+        tune->timeout_id = 0;
+    }
+    if (clock->adjust_timeout_id) {
+        g_source_remove(clock->adjust_timeout_id);
+        clock->adjust_timeout_id = 0;
+    }
+}
+
+void oc_start_tuning(Clock *clock)
+{
+    TimeTuning *tune = clock->tune;
+
+    if (clock->interval == OC_CONFIG_INTERVAL) /* setup active => no changes */
+        return;
+    tune->cnt = 0;
+    tune->changed_once = FALSE;
+    if (tune->timeout_id) {
+        g_source_remove(tune->timeout_id);
+        tune->timeout_id = 0;
+    }
+    tune->timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE
+            , 2*clock->interval, (GSourceFunc)oc_tune_time, clock, NULL);
+}
+
+void oc_init_timer(Clock *clock)
+{
+    clock->interval = OC_BASE_INTERVAL;
+    if (clock->tune == NULL)
+        clock->tune = g_new0(TimeTuning, 1);
+    clock->tune->level = 0;
+    oc_start_timer(clock);
+    oc_start_tuning(clock);
+}
+
 static void oc_update_size(Clock *clock, int size)
 {
     /* keep in sync with systray */
@@ -119,7 +312,8 @@ static void oc_update_size(Clock *clock, int size)
     }
 }
 
-static gboolean popup_program(GtkWidget *widget, gchar *program, Clock *clock)
+static gboolean popup_program(GtkWidget *widget, gchar *program, Clock *clock
+        , guint event_time)
 {
     GdkAtom atom;
     Window xwindow;
@@ -158,6 +352,14 @@ static gboolean popup_program(GtkWidget *widget, gchar *program, Clock *clock)
         return(TRUE);
     }
     else { /* not running, let's try to start it. Need to reset TZ! */
+        static guint prev_event_time = 0; /* prevenst double start (BUG 4096) */
+
+        if (prev_event_time && ((event_time - prev_event_time) < 1000)) {
+            g_message("%s: double start of %s prevented", OC_NAME, program);
+            return(FALSE);
+        }
+            
+        prev_event_time = event_time;
         if (clock->TZ_orig != NULL)  /* we had TZ when we started */
             g_setenv("TZ", clock->TZ_orig, 1);
         else  /* TZ was not set so take it out */
@@ -182,10 +384,12 @@ static gboolean popup_program(GtkWidget *widget, gchar *program, Clock *clock)
 static gboolean on_button_press_event_cb(GtkWidget *widget
         , GdkEventButton *event, Clock *clock)
 {
+    if (event->type != GDK_BUTTON_PRESS) /* double or triple click */
+        return(FALSE); /* ignore */
     if (event->button == 1)
-        return(popup_program(widget, "orage", clock));
+        return(popup_program(widget, "orage", clock, event->time));
     else if (event->button == 2)
-        return(popup_program(widget, "globaltime", clock));
+        return(popup_program(widget, "globaltime", clock, event->time));
 
     return(FALSE);
 }
@@ -212,7 +416,9 @@ static void oc_free_data(XfcePanelPlugin *plugin, Clock *clock)
     if (dlg)
         gtk_widget_destroy(dlg);
     
-    g_source_remove(clock->timeout_id);
+    if (clock->timeout_id) {
+        g_source_remove(clock->timeout_id);
+    }
     g_object_unref(clock->tips);
     g_object_unref(clock->line[0].label);
     g_object_unref(clock->line[1].label);
@@ -228,6 +434,7 @@ static void oc_read_rc_file(XfcePanelPlugin *plugin, Clock *clock)
     const gchar  *ret;
     gchar tmp[100];
     gint i;
+    unsigned int red, green, blue;
 
     if (!(file = xfce_panel_plugin_lookup_rc_file(plugin)))
         return; /* if it does not exist, we use defaults from orage_oc_new */
@@ -242,20 +449,32 @@ static void oc_read_rc_file(XfcePanelPlugin *plugin, Clock *clock)
     clock->fg_set = xfce_rc_read_bool_entry(rc, "fg_set", FALSE);
     if (clock->fg_set) {
         ret = xfce_rc_read_entry(rc, "fg", NULL);
+        /*
         sscanf(ret, "%uR %uG %uB"
                 , (unsigned int *)&clock->fg.red
                 , (unsigned int *)&clock->fg.green
                 , (unsigned int *)&clock->fg.blue);
+                */
+        sscanf(ret, "%uR %uG %uB", &red, &green, &blue);
+        clock->fg.red = red;
+        clock->fg.green = green;
+        clock->fg.blue = blue;
         clock->fg.pixel = 0;
     }
 
     clock->bg_set = xfce_rc_read_bool_entry(rc, "bg_set", FALSE);
     if (clock->bg_set) {
         ret = xfce_rc_read_entry(rc, "bg", NULL);
+        /*
         sscanf(ret, "%uR %uG %uB"
                 , (unsigned int *)&clock->bg.red
                 , (unsigned int *)&clock->bg.green
                 , (unsigned int *)&clock->bg.blue);
+                */
+        sscanf(ret, "%uR %uG %uB", &red, &green, &blue);
+        clock->fg.red = red;
+        clock->fg.green = green;
+        clock->fg.blue = blue;
         clock->bg.pixel = 0;
     }
 
@@ -285,7 +504,7 @@ static void oc_read_rc_file(XfcePanelPlugin *plugin, Clock *clock)
         }
     }
 
-    if (ret = xfce_rc_read_entry(rc, "tooltip", NULL))
+    if ((ret = xfce_rc_read_entry(rc, "tooltip", NULL)))
         g_string_assign(clock->tooltip_data, ret); 
 
     xfce_rc_close(rc);
@@ -424,9 +643,7 @@ Clock *orage_oc_new(XfcePanelPlugin *plugin)
     clock->tips = gtk_tooltips_new();
     g_object_ref(clock->tips);
     gtk_object_sink(GTK_OBJECT(clock->tips));
-    oc_get_time(clock);
-    clock->timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE
-            , 200, (GSourceFunc) oc_get_time, clock, NULL);
+    oc_init_timer(clock);
         
     return(clock);
 }
